@@ -54,7 +54,7 @@ class APNSClientContextFactory(ClientContextFactory):
       log.msg('APNSClientContextFactory ssl_cert_file=%s' % ssl_cert_file)
     else:
       log.msg('APNSClientContextFactory ssl_cert_file={FROM_STRING}')
-    self.ctx = SSL.Context(SSL.SSLv3_METHOD)
+    self.ctx = SSL.Context(SSL.TLSv1_METHOD)
     if 'BEGIN CERTIFICATE' in ssl_cert_file:
       cer = crypto.load_certificate(crypto.FILETYPE_PEM, ssl_cert_file)
       pkey = crypto.load_privatekey(crypto.FILETYPE_PEM, ssl_cert_file)
@@ -241,9 +241,10 @@ class APNSService(service.Service):
 
 class APNSServer(xmlrpc.XMLRPC):
   def __init__(self):
-    self.allowNone = True
     self.app_ids = app_ids
+    self.use_date_time = True
     self.useDateTime = True
+    xmlrpc.XMLRPC.__init__(self, allowNone=True)
   
   def apns_service(self, app_id):
     if app_id not in app_ids:
@@ -269,6 +270,7 @@ class APNSServer(xmlrpc.XMLRPC):
                               'environments are `sandbox` and `production`' % (
                               environment,))
     if not app_id in self.app_ids:
+      # log.msg('provisioning ' + app_id + ' environment ' + environment)
       self.app_ids[app_id] = APNSService(path_to_cert_or_cert, environment, timeout)
   
   def xmlrpc_notify(self, app_id, token_or_token_list, aps_dict_or_list):
@@ -283,7 +285,6 @@ class APNSServer(xmlrpc.XMLRPC):
       Returns:
           None
     """
-    
     d = self.apns_service(app_id).write(
       encode_notifications(
         [t.replace(' ', '') for t in token_or_token_list] 
@@ -326,7 +327,7 @@ def encode_notifications(tokens, notifications):
   if type(notifications) is dict and type(tokens) in (str, unicode):
     tokens, notifications = ([tokens], [notifications])
   if type(notifications) is list and type(tokens) is list:
-    return ''.join(map(lambda y: structify(*y), ((binaryify(t), json.dumps(p, separators=(',',':')))
+    return ''.join(map(lambda y: structify(*y), ((binaryify(t), json.dumps(p, separators=(',',':'), ensure_ascii=False).encode('utf-8'))
                                     for t, p in zip(tokens, notifications))))
 
 def decode_feedback(binary_tuples):
